/*
 * This file is part of vospace-ui
 * Copyright (C) 2021 Istituto Nazionale di Astrofisica
 * SPDX-License-Identifier: GPL-3.0-or-later
 */
package it.inaf.ia2.vospace.ui.controller;

import it.inaf.ia2.vospace.ui.client.VOSpaceClient;
import it.inaf.ia2.vospace.ui.data.Job;
import it.inaf.ia2.vospace.ui.exception.BadRequestException;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;
import net.ivoa.xml.uws.v1.ErrorType;
import net.ivoa.xml.uws.v1.ExecutionPhase;
import net.ivoa.xml.uws.v1.JobSummary;
import net.ivoa.xml.vospace.v2.Param;
import net.ivoa.xml.vospace.v2.Protocol;
import net.ivoa.xml.vospace.v2.StructuredDataNode;
import net.ivoa.xml.vospace.v2.Transfer;
import net.ivoa.xml.vospace.v2.View;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.ByteArrayResource;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestTemplate;

@RestController
public class JobController extends BaseController {

    private static final Logger LOG = LoggerFactory.getLogger(JobController.class);
    
    @Value("${vospace-authority}")
    private String authority;

    @Autowired
    private VOSpaceClient client;

    @Autowired
    private RestTemplate restTemplate;

    @PostMapping(value = "/recall", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Job> startRecallFromTapeJob(@RequestBody List<String> paths) {

        if (paths.isEmpty()) {
            throw new BadRequestException("Received empty list of nodes");
        }

        String target;
        if (paths.size() == 1) {
            target = "vos://" + authority + paths.get(0);
        } else {
            target = createTempListOfFilesNode(paths);
        }

        Transfer transfer = new Transfer();
        transfer.setDirection("pullToVoSpace");
        transfer.setTarget(target);
        Protocol protocol = new Protocol();

        Param param = new Param();
        param.setUri("ia2:node-type");
        param.setValue(paths.size() == 1 ? "single" : "list");
        protocol.getParam().add(param);

        protocol.setUri("ia2:async-recall");
        transfer.getProtocols().add(protocol);

        JobSummary job = client.startTransferJob(transfer);
        
        if (job.getPhase() == ExecutionPhase.QUEUED
                || job.getPhase() == ExecutionPhase.PENDING
                || job.getPhase() == ExecutionPhase.EXECUTING) {
            return ResponseEntity.ok(new Job(job));
        }
        String errorMessage;
        if (job.getPhase() == ExecutionPhase.ERROR) {
            if (job.getErrorSummary() != null) {
                if (job.getErrorSummary().isHasDetail()) {
                    errorMessage = client.getErrorDetail(job.getJobId());
                } else {
                    errorMessage = job.getErrorSummary().getMessage();
                }
                if (job.getErrorSummary().getType() == ErrorType.TRANSIENT) {
                    if (!errorMessage.endsWith(".")) {
                        errorMessage += ".";
                    }
                    errorMessage += " Please retry later.";
                }
            } else {
                errorMessage = "Unexpected error";
            }
        } else {
            LOG.error("Error while executing job {}", job.getJobId());
            errorMessage = "Error while executing job. Job phase is "
                    + job.getPhase() + ". QUEUED, PENDING or EXECUTING expected";
        }
        throw new RuntimeException(errorMessage);
    }

    private String createTempListOfFilesNode(List<String> paths) {

        StructuredDataNode dataNode = createStructuredDataNode(paths);
        client.createNode(dataNode);
        String uploadEndpoint = getTempFileEndpoint(dataNode.getUri());

        String content = String.join("\n", paths);

        upload(uploadEndpoint, content);

        return dataNode.getUri();
    }

    private StructuredDataNode createStructuredDataNode(List<String> paths) {

        List<View> views = new ArrayList<>();
        View view = new View();
        view.setUri("urn:list-of-files");
        views.add(view);

        StructuredDataNode dataNode = new StructuredDataNode();

        String parentPath = getParentPath(paths);
        String newTempFile = ".tmp-" + UUID.randomUUID().toString().replace("-", "") + ".txt";

        dataNode.setUri("vos://" + authority + parentPath + "/" + newTempFile);

        dataNode.setAccepts(views);
        dataNode.setProvides(views);

        return dataNode;
    }

    private String getParentPath(List<String> paths) {
        // All the paths have the same parent, we can choose the first for extracting the path
        String firstPath = paths.get(0);
        return firstPath.substring(0, firstPath.lastIndexOf("/"));
    }

    private String getTempFileEndpoint(String target) {

        Transfer transfer = new Transfer();
        transfer.setDirection("pushToVoSpace");
        transfer.setTarget(target);
        Protocol protocol = new Protocol();
        protocol.setUri("ivo://ivoa.net/vospace/core#httpput");
        transfer.getProtocols().add(protocol);

        return client.getFileServiceEndpoints(transfer).get(0).getEndpoint();
    }

    private void upload(String endpoint, String content) {

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.MULTIPART_FORM_DATA);

        MultiValueMap<String, Object> parts = new LinkedMultiValueMap<>();
        parts.add("file", new MultipartFileResource(content, "list.txt"));

        HttpEntity<MultiValueMap<String, Object>> requestEntity = new HttpEntity<>(parts, headers);

        restTemplate.exchange(endpoint, HttpMethod.PUT, requestEntity, Void.class);
    }

    private class MultipartFileResource extends ByteArrayResource {

        private final String fileName;

        public MultipartFileResource(String content, String fileName) {
            super(content.getBytes());
            this.fileName = fileName;
        }

        @Override
        public String getFilename() {
            return this.fileName;
        }
    }

    @GetMapping(value = "/jobs", produces = MediaType.APPLICATION_JSON_VALUE)
    public List<Job> getJobs() {
        return client.getJobs();
    }
}
