/*
 * This file is part of vospace-ui
 * Copyright (C) 2021 Istituto Nazionale di Astrofisica
 * SPDX-License-Identifier: GPL-3.0-or-later
 */
package it.inaf.ia2.vospace.ui.controller;

import com.fasterxml.jackson.databind.ObjectMapper;
import it.inaf.ia2.vospace.ui.client.VOSpaceClient;
import java.util.Arrays;
import java.util.List;
import net.ivoa.xml.vospace.v2.DataNode;
import static org.junit.jupiter.api.Assertions.assertTrue;
import org.junit.jupiter.api.Test;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.doThrow;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import org.springframework.web.util.NestedServletException;

@SpringBootTest
@AutoConfigureMockMvc
public class NodesControllerTest {

    private static final ObjectMapper MAPPER = new ObjectMapper();

    @MockBean
    private VOSpaceClient client;

    @Autowired
    private MockMvc mockMvc;

    @Test
    public void testListNodesEmpty() throws Exception {

        when(client.getNode(any())).thenReturn(new DataNode());

        mockMvc.perform(get("/nodes"))
                .andExpect(status().isOk());

        verify(client, times(1)).getNode(eq("/"));
    }

    @Test
    public void testListNodesRoot() throws Exception {

        when(client.getNode(any())).thenReturn(new DataNode());

        mockMvc.perform(get("/nodes/"))
                .andExpect(status().isOk());

        verify(client, times(1)).getNode(eq("/"));
    }

    @Test
    public void testListNodesComplexPath() throws Exception {

        when(client.getNode(any())).thenReturn(new DataNode());

        mockMvc.perform(get("/nodes/a/b/c"))
                .andExpect(status().isOk());

        verify(client, times(1)).getNode(eq("/a/b/c"));
    }

    @Test
    public void testDeleteMultipleNodes() throws Exception {

        List<String> paths = Arrays.asList("/a/b/c", "/e/f/g");

        mockMvc.perform(post("/delete")
                .contentType(MediaType.APPLICATION_JSON)
                .content(MAPPER.writeValueAsString(paths)))
                .andExpect(status().isNoContent());

        verify(client, times(1)).deleteNode(eq("/a/b/c"));
        verify(client, times(1)).deleteNode(eq("/e/f/g"));
    }

    @Test
    public void testErrorOnDelete() throws Exception {

        doThrow(new RuntimeException())
                .when(client).deleteNode(any());

        boolean exception = false;
        try {
            mockMvc.perform(post("/delete")
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(MAPPER.writeValueAsString(Arrays.asList("/test"))))
                    .andReturn();
        } catch (NestedServletException ex) {
            exception = true;
        }

        assertTrue(exception);
    }
}
